@@ -53,6 +53,7 @@ static char sccsid[] = "@(#)hash.c	8.9 (Berkeley) 6/16/94";
 #include <assert.h>
 #endif
 
+#define __DBINTERFACE_PRIVATE	/* activate prototypes from db_local.h */
 #include "db_local.h"
 #include "hash.h"
 #include "page.h"
@@ -70,7 +71,7 @@ static void *hash_realloc(SEGMENT **, int, int);
 static int   hash_seq(const DB *, DBT *, DBT *, __uint32_t);
 static int   hash_sync(const DB *, __uint32_t);
 static int   hdestroy(HTAB *);
-static HTAB *init_hash(HTAB *, const char *, HASHINFO *);
+static HTAB *init_hash(HTAB *, const char *, const HASHINFO *);
 static int   init_htab(HTAB *, int);
 #if (BYTE_ORDER == LITTLE_ENDIAN)
 static void  swap_header(HTAB *);
@@ -103,14 +104,20 @@ int hash_accesses, hash_collisions, hash_expansions, hash_overflows;
 /* OPEN/CLOSE */
 
 extern DB *
-__hash_open(file, flags, mode, info, dflags)
-	const char *file;
-	int flags, mode, dflags;
-	const HASHINFO *info;	/* Special directives for create */
+_DEFUN(__hash_open, (file, flags, mode, info, dflags),
+	const char *file _AND
+	int flags _AND
+	int mode _AND
+	int dflags _AND
+	const HASHINFO *info)	/* Special directives for create */
 {
 	HTAB *hashp;
 
+#ifdef __USE_INTERNAL_STAT64
+        struct stat64 statbuf;
+#else
 	struct stat statbuf;
+#endif
 	DB *dbp;
 	int bpages, hdrsize, new_table, nsegs, save_errno;
 
@@ -307,7 +314,7 @@ static HTAB *
 init_hash(hashp, file, info)
 	HTAB *hashp;
 	const char *file;
-	HASHINFO *info;
+	const HASHINFO *info;
 {
 	struct stat statbuf;
 	int nelem;
