@@ -33,7 +33,7 @@ _DEFUN(vprintf, (fmt, ap),
        _CONST char *fmt _AND
        va_list ap)
 {
-  _REENT_SMALL_CHECK_INIT (_stdout_r (_REENT));
+  _REENT_SMALL_CHECK_INIT (_REENT);
   return _vfprintf_r (_REENT, _stdout_r (_REENT), fmt, ap);
 }
 
@@ -45,6 +45,6 @@ _DEFUN(_vprintf_r, (ptr, fmt, ap),
        _CONST char *fmt   _AND
        va_list ap)
 {
-  _REENT_SMALL_CHECK_INIT (_stdout_r (ptr));
+  _REENT_SMALL_CHECK_INIT (ptr);
   return _vfprintf_r (ptr, _stdout_r (ptr), fmt, ap);
 }
