@@ -9,6 +9,7 @@
 int (*__wctomb) (struct _reent *, char *, wchar_t, const char *charset,
 		 mbstate_t *)
 #ifdef __CYGWIN__
+   /* Cygwin starts up in UTF-8 mode. */
     = __utf8_wctomb;
 #else
     = __ascii_wctomb;
@@ -40,7 +41,11 @@ _DEFUN (__ascii_wctomb, (r, s, wchar, charset, state),
   if (s == NULL)
     return 0;
  
+#ifdef __CYGWIN__
+  if ((size_t)wchar >= 0x80)
+#else
   if ((size_t)wchar >= 0x100)
+#endif
     {
       r->_errno = EILSEQ;
       return -1;
